import functools

from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for, current_app
)
from werkzeug.security import check_password_hash, generate_password_hash
from wtforms.validators import StopValidation
from flaskr.db import get_db
from flaskr.db import PostQ_analogy, PostQ_accuracy
from flaskr.template import OptionRequired
from flaskr.util import return_img_stream
import os

bp = Blueprint('analogy_evaluation', __name__, url_prefix='/analogy_evaluation')

from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, SelectField, RadioField
from wtforms.validators import DataRequired, Length


binary_choices = [("0", "Select"), ('yes','yes'),('no','no')]
Likert_5_choices = choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')]

template_list = ["Select Template", "Sign Of", "Always Found At", "Seldom Found At", "Contradict With", "Insufficient"]
template_choices = [("0", "Select a template"), ("1", "Sign Of"), ("2", "Always Found At"), ("3", "Seldom Found At"), ("4", "Contradict With"), ("5", "Insufficient")]

class submit_form(FlaskForm):
    submit_button = SubmitField('Next')

# class Conceptform(FlaskForm):
#     template = SelectField('Select a template to write the analogy.', choices=template_choices, validators=[OptionRequired()], render_kw={"onchange":"changefield('template',explanation,'/changeselectfield/')"})
#     # template = RadioField('Select a template to write the analogy.', choices=template_choices, validators=[DataRequired(), render_kw={"onchange":"changefield('template',explanation,'/changeselectfield/')"}])
#     concept_1 = StringField('ConceptA', validators=[DataRequired(), Length(max=500)])
#     concept_2 = StringField('ConceptB', validators=[DataRequired(), Length(max=500)])
#     concept_3 = StringField('ConceptC (Only for Insufficient Template)', validators=[Length(max=500), ConceptCRequired_places()])
#     submit_button = SubmitField('Submit')

class Evaluationform(FlaskForm):
    item_1 = RadioField('How helpful is this explanation for you to understand the target sentence? -> (1: not helpful at all to 5: very helpful)', choices=Likert_5_choices, validators=[DataRequired()])
    item_2 = RadioField('Look at the alignment from explanation to target sentence. How good can you align the properties of the explanation ceonpts to the properties of the concepts in the target sentence? -> (1: not at all good to 5: extremely good)', choices=Likert_5_choices, validators=[DataRequired()])
    item_3 = RadioField('How similar do you perceive the relationship between concepts in the explanation and the relationship between concepts in the target sentence? -> (1: not at all similar to 5: extremely similar)', choices=Likert_5_choices, validators=[DataRequired()])
    item_4 = RadioField('Do you think the explanation is simple enough for others to understand? -> (1: not at all simple 5: very simple)', choices=Likert_5_choices, validators=[DataRequired()])
    item_5 = RadioField('How familiar are you with the concepts in the explanation? -> (1: not at all familiar to 5: very familiar)', choices=Likert_5_choices, validators=[DataRequired()])
    item_6 = RadioField('How well can the explanation be used in other contexts? -> (1: not at all good to 5: extremely good)', choices=Likert_5_choices, validators=[DataRequired()])
    item_7 = SelectField('Do you think this explanation causes lead to more than single interpretation?', choices=binary_choices, validators=[OptionRequired()], render_kw={"onchange":"hideTextField()"})
    item_8 = StringField('What misunderstanding do you think the explanation cause?', validators=[Length(max=500)])
    #, render_kw={"id":"item_8"}
    submit_button = SubmitField('Submit')


@bp.route('/instruction', methods=('GET', 'POST'))
def instruction():
    # show the user instruction for whole task
    form = submit_form()
    if request.method == 'POST':
        # we may record the time user finish reading instructions
        return redirect(url_for("analogy_evaluation.evaluation_task"))
    return render_template('analogy_evaluation/instruction.html', form=form)

class evaluation_sample(object):
    def __init__(self, explanation_concept1, explanation_concept2, template, target_concept1, target_concept2, explanation_concept3=None):
        self.explanation_concept1 = explanation_concept1
        self.explanation_concept2 = explanation_concept2
        self.explanation_concept3 = explanation_concept3
        self.template = template
        self.target_concept1 = target_concept1
        self.target_concept2 = target_concept2
        if self.template == "Insufficient":
            assert self.explanation_concept3 is not None

    def _obtain_explanation(self):
        analogy_template = {
            "Sign Of": f"It’s like the <strong>{self.explanation_concept1}</strong> of <strong>{self.explanation_concept2}</strong>.",
            "Always Found At": f"It’s like <strong>{self.explanation_concept1}</strong> can often be found in <strong>{self.explanation_concept2}</strong>.",
            "Seldom Found At": f"It’s like <strong>{self.explanation_concept1}</strong> can hardly be found in <strong>{self.explanation_concept2}</strong>.",
            "Contradict With": f"It is like you can’t find <strong>{self.explanation_concept1}</strong> in the <strong>{self.explanation_concept2}</strong>.", 
            "Insufficient": f"As you can find <strong>{self.explanation_concept1}</strong> in both <strong>{self.explanation_concept2}</strong> and <strong>{self.explanation_concept3}</strong>, you can’t determine where it is if you see <strong>{self.explanation_concept1}</strong>."
        }
        return analogy_template[self.template]

    def _obtain_target_sentence(self):
        return f"The relevance relationship of observing concept [<strong>{self.target_concept1}</strong>] to give a label [<strong>{self.target_concept2}</strong>]."

    def get_sentences(self):
        explanation = self._obtain_explanation()
        target_sentence = self._obtain_target_sentence()
        return target_sentence, explanation

    def serize(self):
        # return jsonify(Loan_ID=self.Loan_ID, Education=self.Education, ApplicantIncome=self.ApplicantIncome,
        #  CoapplicantIncome=self.CoapplicantIncome, LoanAmount=self.LoanAmount, Loan_Amount_Term=self.Loan_Amount_Term,
        #  Credit_History=self.Credit_History, Property_Area=self.Property_Area, Loan_Status=self.Loan_Status)
        # Use rand_ID to show, to avoid users access to real Loan_ID
        return {
            "explanation_concept1": self.explanation_concept1,
            "explanation_concept2": self.explanation_concept2,
            "explanation_concept3": self.explanation_concept3,
            "template": self.template,
            "target_concept1": self.target_concept1,
            "target_concept2": self.target_concept2,
        }

@bp.route('/evaluation_task', methods=['GET', 'POST'])
def evaluation_task():
    form = Evaluationform()
    # task_id = session['task_id']
    task_id = "try"
    db = get_db()
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        answer_dict = {"evaluator_id": user_id}
        template = form.template.data
        for i in range(8):
            tp_key = 'item_{}'.format(i + 1)
            answer_dict[tp_key] = getattr(form, tp_key).data
        error = None
        from flaskr.db import AnalogyEvaluation
        q = db.query(AnalogyEvaluation.evaluator_id).filter(AnalogyEvaluation.evaluator_id==evaluator_id, AnalogyEvaluation.task_id==task_id)
        exist_flag = db.query(q.exists()).scalar()
        if not exist_flag:
            ut_obj = AnalogyEvaluation(**answer_dict)
            db.add(ut_obj)
            db.commit()
        return redirect(url_for("hello"))
    # template_list = ["Select Template", "Sign Of", "Always Found At", "Seldom Found At", "Contradict With", "Insufficient"]
    # evaluation_task_list[task_id]
    # Obtain data with task_id
    concept = "chocolate and ice cream"
    prediction = "high fat"
    concept_1 = "computer and printer"
    concept_2 = "office"
    concept_3 = None
    template_name = "Always Found At"
    # target_concept3 = session[??]
    current_case = evaluation_sample(target_concept1=concept, target_concept2=prediction, template=template_name, explanation_concept1=concept_1, explanation_concept2=concept_2, explanation_concept3=concept_3)
    target_sentence, explanation = current_case.get_sentences()
    current_data = current_case.serize()
    return render_template('analogy_evaluation/evaluation_task.html', target_sentence=target_sentence, explanation=explanation, data=current_data, form=form)








