import functools

from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for, current_app, jsonify
)
from werkzeug.security import check_password_hash, generate_password_hash
from wtforms.validators import StopValidation
from flaskr.db import get_db
from flaskr.db import PostQ_analogy, PostQ_accuracy
from flaskr.db import AnalogyAnnotation, UserBehavior
from flaskr.template import OptionRequired, ConceptCRequired_places, ConceptPlaceForbid, ConceptCRequired_calorie, ConceptCalorieForbid
from flaskr.util import return_img_stream
import os
import pandas as pd
import numpy as np
bp = Blueprint('analogy_generation', __name__, url_prefix='/analogy_generation')

from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, SelectField, RadioField
from wtforms.validators import DataRequired, Length

class GenerationData(object):

    def __init__(self, tp_task):
        task_id,image_path,concept,prediction = tp_task
        self.task_id = task_id
        self.image_path = image_path
        self.concept = concept
        self.prediction = prediction

    @staticmethod
    def deal_int(value):
        if value is not None:
            out_str = "%d"%(value)
        else:
            out_str = "0"
        return out_str

    def print_generation_task(self):
        task_str = ""
        task_str += "Task %s"%(self.task_id)
        task_str += ", concept: {}".format(self.concept)
        task_str += ", with label %s"%(self.prediction)
        return task_str

    def serize(self):
        # return jsonify(Loan_ID=self.Loan_ID, Education=self.Education, ApplicantIncome=self.ApplicantIncome,
        #  CoapplicantIncome=self.CoapplicantIncome, LoanAmount=self.LoanAmount, Loan_Amount_Term=self.Loan_Amount_Term,
        #  Credit_History=self.Credit_History, Property_Area=self.Property_Area, Loan_Status=self.Loan_Status)
        # Use rand_ID to show, to avoid users access to real Loan_ID
        return {
            "task_id": self.task_id,
            "image_path": self.image_path,
            "concept": self.concept,
            "prediction": self.prediction
        }

def load_generation_tasks(filename="flaskr/static/data/calorie_task.csv"):
    df = pd.read_csv(filename)
    data_dict = {}
    task_ID_list = []
    for i in range(len(df)):
        tp_data = GenerationData(df.iloc[i,:].tolist())
        data_dict[tp_data.task_id] = tp_data
        task_ID_list.append(tp_data.task_id)
    # print(data_dict['T003'].print_generation_task())
    return task_ID_list, data_dict

task_ID_list, generation_task_dict = load_generation_tasks("flaskr/static/data/balanced_task_10.csv")

binary_choices = [("0", "Select"), ('yes','yes'),('no','no')]
Likert_5_choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')]

from flaskr.template import template_list, template_choices, id2template

class submit_form(FlaskForm):
    submit_button = SubmitField('Next')

class Conceptform_places(FlaskForm):
    template = SelectField('First select a template to write the analogy.', choices=template_choices, validators=[OptionRequired()], render_kw={"onchange":"changefield('template',explanation,'/analogy_generation/changeselectfield/')"})
    # template = RadioField('Select a template to write the analogy.', choices=template_choices, validators=[DataRequired(), render_kw={"onchange":"changefield('template',explanation,'/changeselectfield/')"}])
    concept_1 = StringField('A', validators=[DataRequired(), ConceptPlaceForbid(), Length(max=100)])
    concept_2 = StringField('B', validators=[DataRequired(), ConceptPlaceForbid(), Length(max=100)])
    concept_3 = StringField('C', validators=[Length(max=100), ConceptCRequired_places()])
    submit_button = SubmitField('Submit')

class Conceptform_calorie(FlaskForm):
    template = SelectField('First select a template to write the analogy.', choices=template_choices, validators=[OptionRequired()], render_kw={"onchange":"changefield('template',explanation,'/analogy_generation/changeselectfield/')"})
    # template = RadioField('Select a template to write the analogy.', choices=template_choices, validators=[DataRequired(), render_kw={"onchange":"changefield('template',explanation,'/changeselectfield/')"}])
    concept_1 = StringField('A', validators=[DataRequired(), ConceptCalorieForbid(), Length(max=100)])
    concept_2 = StringField('B', validators=[DataRequired(), ConceptCalorieForbid(), Length(max=100)])
    concept_3 = StringField('C', validators=[Length(max=100), ConceptCRequired_calorie()])
    submit_button = SubmitField('Submit')

@bp.route('/changeselectfield/', methods=['GET', 'POST'])
def changeselectfield():
    analogy_template = {
        "Select a template": "",
        "Definite Sign Of": f"This is like <strong>[A]</strong> is definitely a sign of <strong>[B]</strong>.",
        "Typically Associated With": f"This is like <strong>[A]</strong> can typically be associated with <strong>[B]</strong>, but it’s also possible to associate <strong>[A]</strong> with <strong>[C]</strong>.",
        "Seldom Found At": f"This is like <strong>[A]</strong> can seldom be found in <strong>[B]</strong>.",
        "Contradict With": f"This is similar to how one cannot find <strong>[A]</strong> in <strong>[B]</strong>.", 
        "Insufficient": f"This is similar to how we can find <strong>[A]</strong> in both <strong>[B]</strong> and <strong>[C]</strong>, you can’t determine where it is if you see <strong>[A]</strong>.",
        "Irrelevant": f"This is similar to to how <strong>[A]</strong> is irrelevant for recognising a <strong>[B]</strong>."
    }
    if request.method == "POST":
        data = request.get_json()
        name = data['name']
        # song = analogy_template[name]
        if name == "Select a template":
            return jsonify("")
        explanation = analogy_template[name]
        return jsonify(explanation)
    else:
        return {}

@bp.route('/instruction', methods=('GET', 'POST'))
def instruction():
    # show the user instruction for whole task
    form = submit_form()
    image_path = "flaskr/static/images/calorie/meal_images/tiramisu-1-deductive.png"
    img_stream = return_img_stream(image_path)
    if request.method == 'POST':
        # we may record the time user finish reading instructions
        if session['task_type'] == "calorie":
            return redirect(url_for("analogy_generation.calorie_task"))
        else:
            return redirect(url_for("analogy_generation.places_task"))
        # if session['first_task'] == "calorie":
        #     return redirect(url_for("analogy_generation.calorie_task"))
        # else:
        #     return redirect(url_for("analogy_generation.places_task"))
    return render_template('analogy_generation/instruction.html', form=form, img_stream=img_stream)

@bp.route('/calorie_task', methods=['GET', 'POST'])
def calorie_task():
    if request.method == 'POST':
        return redirect(url_for("analogy_generation.calorie_example"))
    return render_template('calorie/task_intro.html')

@bp.route('/calorie_example', methods=['GET', 'POST'])
def calorie_example():
    # colours = ["question", "red", "blue"]
    image_path = "flaskr/static/images/calorie/meal_images/tiramisu-1-deductive.png"
    # print(os.path.isfile(image_path))
    img_stream = return_img_stream(image_path)
    # template_list = ["Select Template", "Totally relevant - Sign Of", "Highly relevant - Always Found At", "Less relevant - Seldom Found At", "Contradiction - Contradict With", "Less relevant - Insufficient"]
    # concept_list = ["avacado", "chocolate", "chicken", "vegetable", "bread"]
    # prediction = "high calorie"
    # model_sent = ""
    # for concept in concept_list:
    #     model_sent += f"Follow the templates to formulate the relevance relationship of observing concept [<strong>{concept}</strong>] to give a label [<strong>{prediction}</strong>].<br>"
    if request.method == 'POST':
        return redirect(url_for("analogy_generation.generation_task"))
    return render_template('calorie/example.html', img_stream=img_stream)

@bp.route('/places_task', methods=['GET', 'POST'])
def places_task():
    if request.method == 'POST':
        return redirect(url_for("analogy_generation.places_example"))
    return render_template('places365/task_intro.html')

@bp.route('/places_example', methods=['GET', 'POST'])
def places_example():
    # colours = ["question", "red", "blue"]
    image_path = "flaskr/static/images/places365/Places365_val_00000244.jpg"
    # print(os.path.isfile(image_path))
    img_stream = return_img_stream(image_path)
    # label_list = ["bathroom", "bedroom", "conference room", "dining room", "hospital room", "living room"]
    if request.method == 'POST':
        return redirect(url_for("analogy_generation.generation_task"))
    return render_template('places365/example.html', img_stream=img_stream)

def save_annotation(annotator_id, task_id, template, concept_1, concept_2, concept_3):
    db = get_db()
    error = None
    q = db.query(AnalogyAnnotation.annotator_id).filter(AnalogyAnnotation.annotator_id==annotator_id, AnalogyAnnotation.task_id==task_id)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = AnalogyAnnotation(annotator_id=annotator_id, task_id=task_id, template=template,
         concept_1=concept_1, concept_2=concept_2, concept_3=concept_3)
        db.add(ut_obj)
        db.commit()

@bp.route('/', methods=['GET', 'POST'])
def user_port():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    # task_types = ["calorie", "places"]
    # first_task = np.random.choice(task_types, 1)[0]
    # session['first_task'] = first_task
    # analogy_type = 'train'
    # session['analogy_type'] = "weather"
    session['task_type'] = "calorie"
    
    # tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = task_ID_list[:6]
    session['check_pass'] = False
    current_app.logger.info("User {} log in".format(user_id))
    return redirect(url_for("analogy_generation.instruction"))

@bp.route('/user_port_1/', methods=['GET', 'POST'])
def user_port_1():
    user_id = request.args.get('PROLIFIC_PID')
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    # task_types = ["calorie", "places"]
    # first_task = np.random.choice(task_types, 1)[0]
    session['task_type'] = "calorie"
    # analogy_type = 'train'
    # session['analogy_type'] = "weather"
    
    # tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = task_ID_list[:6]
    session['check_pass'] = False
    current_app.logger.info("User {} log in".format(user_id))
    return redirect(url_for("analogy_generation.instruction"))

@bp.route('/user_port_2/', methods=['GET', 'POST'])
def user_port_2():
    user_id = request.args.get('PROLIFIC_PID')
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    # task_types = ["calorie", "places"]
    # first_task = np.random.choice(task_types, 1)[0]
    session['task_type'] = "places"
    # analogy_type = 'train'
    # session['analogy_type'] = "weather"
    
    # tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = task_ID_list[6:]
    session['check_pass'] = False
    current_app.logger.info("User {} log in".format(user_id))
    return redirect(url_for("analogy_generation.instruction"))

def check_analogy_existence(annotator_id, template, concept_1, concept_2, concept_3):
    db = get_db()
    error = None
    from flaskr.db import AnalogyAnnotation
    if template in ["Typically Associated With", "Insufficient"]:
        q = db.query(AnalogyAnnotation.annotator_id).filter(AnalogyAnnotation.annotator_id==annotator_id,
            AnalogyAnnotation.concept_1==concept_1, AnalogyAnnotation.concept_2==concept_2, AnalogyAnnotation.concept_3==concept_3)
    else:
        q = db.query(AnalogyAnnotation.annotator_id).filter(AnalogyAnnotation.annotator_id==annotator_id,
            AnalogyAnnotation.concept_1==concept_1, AnalogyAnnotation.concept_2==concept_2)
    exist_flag = db.query(q.exists()).scalar()
    return exist_flag

@bp.route('/usehints', methods=['POST'])
def usehints():
    db = get_db()
    user_id = session['user_id']
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    # current_task = session['task_type']
    tp_task_ID = task_ID_list[task_id]
    # current_task = "calorie"
    # if session['first_task'] == "calorie":
    #     tp_task_ID = task_ID_list[task_id]
    #     if task_id >= 6:
    #         current_task = "places"
    # else:
    #     if task_id + 6 < 12:
    #         tp_task_ID = task_ID_list[task_id + 6]
    #         current_task = "places"
    #     else:
    #         tp_task_ID = task_ID_list[task_id - 6]
    event_desc = tp_task_ID + "|hintdomain"
    user_behavior = "click"
    q = db.query(UserBehavior.user_id).filter(UserBehavior.user_id==user_id, UserBehavior.event_desc==event_desc)
    exist_flag = db.query(q.exists()).scalar()
    current_app.logger.info("User {} clicked this button, existence: {}".format(user_id, exist_flag))
    if not exist_flag:
        ut_obj = UserBehavior(user_id=user_id, event_desc=event_desc, user_behavior=user_behavior)
        db.add(ut_obj)
        db.commit()
    return ('', 204)

@bp.route('/generation_task', methods=['GET', 'POST'])
def generation_task():
    current_task = session['task_type']
    if current_task == "calorie":
        form = Conceptform_calorie()
    else:
        form = Conceptform_places()
    # colours = ["question", "red", "blue"]
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    # current_task = "calorie"
    
    tp_task_ID = task_ID_list[task_id]
    # if session['first_task'] == "calorie":
    #     tp_task_ID = task_ID_list[task_id]
    #     if task_id >= 6:
    #         current_task = "places"
    # else:
    #     if task_id + 6 < 12:
    #         tp_task_ID = task_ID_list[task_id + 6]
    #         current_task = "places"
    #     else:
    #         tp_task_ID = task_ID_list[task_id - 6]
    if request.method == 'POST' and form.validate_on_submit():
        annotator_id = session['user_id']
        template = id2template[form.template.data]
        concept_1 = form.concept_1.data.lower()
        concept_2 = form.concept_2.data.lower()
        if template in ["Typically Associated With", "Insufficient"]:
            concept_3 = form.concept_3.data.lower()
        else:
            concept_3 = "None"
        exist_flag = check_analogy_existence(annotator_id=annotator_id, template=template, 
            concept_1=concept_1, concept_2=concept_2, concept_3=concept_3)
        if exist_flag:
            flash("You have generated the same explanation, please come up with a new one!")
            return redirect(url_for('analogy_generation.generation_task'))
        save_annotation(annotator_id=annotator_id, task_id=tp_task_ID, template=template,
            concept_1=concept_1, concept_2=concept_2, concept_3=concept_3)
        # return redirect(url_for("analogy.analogy_evaluation"))
        if task_id < len(task_ID_list) -1:
            # inser before redirection to next page
            session['task_id'] += 1
            # introduce places example
            # if session['task_id'] == 6:
            #     if session['first_task'] == "calorie":
            #         return redirect(url_for('analogy_generation.places_task'))
            #     else:
            #         return redirect(url_for('analogy_generation.calorie_task'))
            # start next task
            return redirect(url_for('analogy_generation.generation_task'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            # return render_template('thanks.html', reason_str = "You have finished all tasks.")
            return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
        # return redirect(url_for("analogy_generation.analogy_evaluation"))
    # image_path = task2imgpath[task_id]
    # print(os.path.isfile(image_path))
    # img_stream = return_img_stream(image_path)
    generation_data = generation_task_dict[tp_task_ID].serize()
    image_path = generation_data['image_path']
    concept = generation_data['concept']
    prediction = generation_data['prediction']
    img_stream = return_img_stream(image_path)
    # template_list = ["Select Template", "Sign Of", "Always Found At", "Seldom Found At", "Contradict With", "Insufficient"]
    # concept = "broken door"
    # prediction = "severe damage"
    model_sent = f"Follow the templates to formulate the relevance relationship of observing concept [<strong>{concept}</strong>] to give a label [<strong>{prediction}</strong>]."
    # if task_id < 7:
    #     task_index = task_id+1
    #     task_total = 7
    # else:
    #     task_index = task_id + 1 - 7
    #     task_total = len(task_ID_list) - 7
    return render_template('analogy_generation/generation_task.html', task_index=task_id+1, current_task=current_task, task_total=len(task_ID_list), model_sent=model_sent, form=form, img_stream=img_stream)






