import functools

from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for, current_app
)
from werkzeug.security import check_password_hash, generate_password_hash
from wtforms.validators import StopValidation
from flaskr.db import get_db, UserTask
from flaskr.template import OptionRequired
from flaskr.util import return_img_stream
import os
import pandas as pd
import numpy as np
bp = Blueprint('bird_task', __name__, url_prefix='/bird_task')

from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, SelectField, RadioField
from wtforms.validators import DataRequired, Length

birdclassname = {
    "AG": "American Goldfinch",
    "LeG": "Lesser Goldfinch",
    "LaG": "Lawrence Goldfinch",
    "GW": "Gila Woodpecker",
    "DW": "Downy Woodpecker",
    "HW": "Hairy Woodpecker"
}

class BirdDataset(object):

    def __init__(self, tp_task):
        task_id,image_path,ground_truth,prediction,concepts = tp_task
        self.task_id = task_id
        self.image_path = image_path
        self.ground_truth = birdclassname[ground_truth]
        self.prediction = birdclassname[prediction]
        self.concepts = concepts

    @staticmethod
    def deal_int(value):
        if value is not None:
            out_str = "%d"%(value)
        else:
            out_str = "0"
        return out_str

    def print_generation_task(self):
        task_str = ""
        task_str += "Task %s"%(self.task_id)
        task_str += ", concepts: {}".format(self.concepts)
        task_str += ", with label %s"%(self.prediction)
        task_str += ", with ground truth %s"%(self.ground_truth)
        return task_str

    def serize(self):
        return {
            "task_id": self.task_id,
            "image_path": self.image_path,
            "ground_truth": self.ground_truth,
            "prediction": self.prediction,
            "concepts": self.concepts
        }

def load_generation_tasks(filename="flaskr/static/data/bird_task.csv"):
    df = pd.read_csv(filename)
    data_dict = {}
    task_ID_list = []
    for i in range(len(df)):
        tp_data = BirdDataset(df.iloc[i,:].tolist())
        data_dict[tp_data.task_id] = tp_data
        task_ID_list.append(tp_data.task_id)
    # print(data_dict['T003'].print_generation_task())
    return task_ID_list, data_dict

task_ID_list, bird_task_dict = load_generation_tasks("flaskr/static/data/bird_6class.csv")

from flaskr.template import template_list, template_choices, id2template

class submit_form(FlaskForm):
    submit_button = SubmitField('Next')

bird_species = ['American Goldfinch', 'Lesser Goldfinch', 'Lawrence Goldfinch', 'Gila Woodpecker', 'Downy Woodpecker', 'Hairy Woodpecker']
global_explanations = {
    'American Goldfinch': "yellow body (belly, breast, throat, side, back); black head; orange beak; black and white wing; black and white tail",
    'Lesser Goldfinch': "yellow body (belly, breast, throat, side), black body (back); black head; grey beak; black and white AND wing; (black and white OR yellow) tail", 
    'Lawrence Goldfinch': "yellow body (belly, breast), (black OR grey) body (throat), grey body (side, back); black AND/OR grey head, (black OR grey) head; grey beak; (black, grey, white and yellow) wing; grey tail, (black, white and grey) tail", 
    'Gila Woodpecker': "brown body (breast, belly, throat, side, back); brown head, red head; black and white wing; black and white tail",
    'Downy Woodpecker': "black and white body (back), white body (belly, breast, throat, side); (white, black and red) head; grey beak; black wing, black and white wing; black tail, black and white tail", 
    'Hairy Woodpecker': "black and white body (back), white body (belly, breast, throat, side); (white, black and red) head; grey beak; black wing, black and white wing; black and white tail"
}
bird_choices = [(bird_species[index], "{} : {}".format(bird_species[index], global_explanations[bird_species[index]])) for index in range(len(bird_species))]

class BirdForm(FlaskForm):
    question = RadioField('Given the image, which specie do you think this bird belong to?', choices=bird_choices, validators=[DataRequired()])
    submit_button = SubmitField('Submit')


@bp.route('/instruction', methods=('GET', 'POST'))
def instruction():
    # show the user instruction for whole task
    form = submit_form()
    # image_path = "flaskr/static/images/bird/lesser_goldfinch_1.jpg"
    # img_stream = return_img_stream(image_path)
    if request.method == 'POST':
        return redirect(url_for("bird_task.bird_task_intro"))
    return render_template('bird_task/instruction.html', form=form)

@bp.route('/bird_task_intro', methods=['GET', 'POST'])
def bird_task_intro():
    if request.method == 'POST':
        return redirect(url_for("bird_task.bird_example"))
    return render_template('bird_task/task_intro.html')

@bp.route('/bird_example', methods=['GET', 'POST'])
def bird_example():
    if request.method == 'POST':
        return redirect(url_for("bird_task.bird_task_base"))
    image_path = "flaskr/static/images/bird/lesser_goldfinch_2.jpg"
    # print(os.path.isfile(image_path))
    img_stream = return_img_stream(image_path)
    return render_template('bird_task/example.html', img_stream=img_stream)

@bp.route('/', methods=['GET', 'POST'])
def user_port():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    exp_conditions = ["control", "ARCH", "ARCHIE"]
    condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = condition
    
    # tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = task_ID_list
    session['check_pass'] = False
    current_app.logger.info("User {} log in".format(user_id))
    return redirect(url_for("bird_task.instruction"))

@bp.route('/post_questionnaire', methods=['GET', 'POST'])
def post_questionnaire():
    form = Conceptform()

def save_annotation(user_id, task_id, answer_type, user_choice):
    db = get_db()
    error = None
    q = db.query(UserTask.user_id).filter(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = UserTask(user_id=user_id, task_id=task_id, answer_type=answer_type, choice=user_choice)
        db.add(ut_obj)
        db.commit()

@bp.route('/bird_task_base', methods=['GET', 'POST'])
def bird_task_base():
    form = BirdForm()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        condition = session['condition']
        user_choice = form.question.data.lower()
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="base", user_choice=user_choice)
        if task_id < len(task_ID_list) -1:
            # start second stage
            if condition == "ARCHIE":
                return redirect(url_for('bird_task.bird_task_archie'))
            elif condition == "ARCH":
                return redirect(url_for('bird_task.bird_task_arch'))
            else:
                return redirect(url_for('bird_task.bird_task_control'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return render_template('thanks.html', reason_str = "You have finished all tasks.")
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    bird_data = bird_task_dict[tp_task_ID].serize()
    image_path = bird_data['image_path']
    img_stream = return_img_stream(image_path)
    return render_template('bird_task/bird_task.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, img_stream=img_stream, advice="None", explanation_sent="None")

@bp.route('/bird_task_control', methods=['GET', 'POST'])
def bird_task_control():
    form = BirdForm()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data.lower()
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="control", user_choice=user_choice)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('bird_task.bird_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return render_template('thanks.html', reason_str = "You have finished all tasks.")
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    bird_data = bird_task_dict[tp_task_ID].serize()
    image_path = bird_data['image_path']
    prediction = bird_data['prediction']
    img_stream = return_img_stream(image_path)
    return render_template('bird_task/bird_task.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, img_stream=img_stream, advice=prediction, explanation_sent="None")

@bp.route('/bird_task_arch', methods=['GET', 'POST'])
def bird_task_arch():
    form = BirdForm()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data.lower()
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="arch", user_choice=user_choice)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('bird_task.bird_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return render_template('thanks.html', reason_str = "You have finished all tasks.")
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    bird_data = bird_task_dict[tp_task_ID].serize()
    image_path = bird_data['image_path']
    concepts = bird_data['concepts']
    prediction = bird_data['prediction']
    explanation_sent = f"Given this image, AI system observed following concepts: [<strong>{concepts}</strong>]. And AI system provides advice: [<strong>{prediction}</strong>]."
    img_stream = return_img_stream(image_path)
    return render_template('bird_task/bird_task.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, img_stream=img_stream, advice=prediction, explanation_sent=explanation_sent)

@bp.route('/bird_task_archie', methods=['GET', 'POST'])
def bird_task_archie():
    form = BirdForm()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data.lower()
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archie", user_choice=user_choice)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('bird_task.bird_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return render_template('thanks.html', reason_str = "You have finished all tasks.")
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    bird_data = bird_task_dict[tp_task_ID].serize()
    image_path = bird_data['image_path']
    concepts = bird_data['concepts']
    prediction = bird_data['prediction']
    explanation_sent = f"Given this image, AI system observed following concepts: [<strong>{concepts}</strong>]. And AI system provides advice: [<strong>{prediction}</strong>]."
    explanation_sent += " It's like (analogy-based explanation)"
    img_stream = return_img_stream(image_path)
    return render_template('bird_task/bird_task.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, img_stream=img_stream, advice=prediction, explanation_sent=explanation_sent)





