import sqlite3

import click
from flask import current_app, g
from flask.cli import with_appcontext
import datetime
from sqlalchemy import create_engine
from sqlalchemy_utils import database_exists, create_database
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Table, Column, Integer, String, MetaData, DateTime


user_id = "root"
user_password = "hegaole945"
dbname = "Analogy"

# Create engine through sqlalchemy
# engine = create_engine("mysql+pymysql://%s:%s@localhost:3306/%s" %(user_id,user_password,dbname),echo=False)
engine = create_engine("mysql+pymysql://%s:%s@localhost:3306/%s" %(user_id,user_password,dbname),echo=False)

# Create database if not exists
if not database_exists(engine.url): 
    create_database(engine.url)         # Create database if it doesn't exist.


def get_db():
    # create a session
    if 'db' not in g:
        DBSession = sessionmaker(bind=engine)

        # 创建session对象:
        g.db = DBSession()

    return g.db

def get_engine():
    return engine

def close_db(e=None):
    # close the session
    db = g.pop('db', None)

    if db is not None:
        db.close()

Base = declarative_base()

class AnalogyAnnotation(Base):
    # table name
    __tablename__ = 'AnalogyAnnotation'

    # table structure
    annotator_id = Column(String(30), primary_key=True)
    task_id = Column(String(30), primary_key=True)
    concept_1 = Column(String(100))
    concept_2 = Column(String(100))
    concept_3 = Column(String(100))
    template = Column(String(30))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        init_str = "<annotator_id:{}, task_id:{}".format(self.annotator_id, self.task_id)
        init_str += " template: {}".format(self.template)
        for i in range(1, 4):
            init_str += " concept_{}: {}".format(i, get_attr(self, 'concept_%d'%(i)))
        return init_str

class AnalogyEvaluation(Base):
    # table name
    __tablename__ = 'AnalogyEvaluation'

    # table structure
    evaluator_id = Column(String(30), primary_key=True)
    task_id = Column(String(30), primary_key=True)
    item_1 = Column(String(20))
    item_2 = Column(String(20))
    item_3 = Column(String(20))
    item_4 = Column(String(20))
    item_5 = Column(String(20))
    item_6 = Column(String(20))
    item_7 = Column(String(20))
    item_8 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        init_str = "<evaluator_id:{}, task_id:{}".format(self.evaluator_id, self.annotator_id, self.task_id)
        for i in range(1, 9):
            init_str += " item_{}: {}".format(i, get_attr(self, 'item_%d'%(i)))
        return init_str


# Define Pre questionnaire table:
class PreQ(Base):
    # table name
    __tablename__ = 'preq'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    answer_2 = Column(String(20))
    answer_3 = Column(String(20))
    answer_4 = Column(String(20))
    answer_5 = Column(String(20))
    answer_6 = Column(String(20))
    answer_7 = Column(String(20))
    answer_8 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        init_str = "<user_id:{}".format(user_id)
        for i in range(1, 9):
            init_str += " answer_{}: {}".format(i, get_attr(self, 'answer_%d'%(i)))
        return init_str
        # return "<user_id:{0} answer_1:{1} answer_2:{2} answer_3:{3} answer_4:{4} answer_5:{5} answer_6:{6} answer_7:{7} answer_8:{8}>".format(self.user_id, self.answer_1,
        #  self.answer_2, self.answer_3, self.answer_4, self.answer_5, self.answer_6, self.answer_7, self.answer_8)

# Define Pre questionnaire table:
# More info here: https://ati-scale.org
class ATI_PreQ (Base):
    # table name
    __tablename__ = 'ATI_preq'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    answer_2 = Column(String(20))
    answer_3 = Column(String(20))
    answer_4 = Column(String(20))
    answer_5 = Column(String(20))
    answer_6 = Column(String(20))
    answer_7 = Column(String(20))
    answer_8 = Column(String(20))
    answer_9 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        init_str = "<user_id:{}".format(user_id)
        for i in range(1, 10):
            init_str += " answer_{}: {}".format(i, get_attr(self, 'answer_%d'%(i)))
        return init_str

class Analogy_PreQ(Base):
    # table name
    __tablename__ = 'Analogy_preq'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    answer_2 = Column(String(20))
    answer_3 = Column(String(20))
    answer_4 = Column(String(20))
    answer_5 = Column(String(20))
    answer_6 = Column(String(20))
    answer_7 = Column(String(20))
    answer_8 = Column(String(20))
    answer_9 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        init_str = "<user_id:{}".format(user_id)
        for i in range(1, 10):
            init_str += " answer_{}: {}".format(i, get_attr(self, 'answer_%d'%(i)))
        return init_str

# Define Trust in Automation Post questionnaire table:
# https://github.com/moritzkoerber/TiA_Trust_in_Automation_Questionnaire
class TiA_PostQ(Base):
    # table name
    __tablename__ = 'TiA_postq'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    answer_2 = Column(String(20))
    answer_3 = Column(String(20))
    answer_4 = Column(String(20))
    answer_5 = Column(String(20))
    answer_6 = Column(String(20))
    answer_7 = Column(String(20))
    answer_8 = Column(String(20))
    answer_9 = Column(String(20))
    answer_10 = Column(String(20))
    answer_11 = Column(String(20))
    answer_12 = Column(String(20))
    answer_13 = Column(String(20))
    answer_14 = Column(String(20))
    answer_15 = Column(String(20))
    answer_16 = Column(String(20))
    answer_17 = Column(String(20))
    answer_18 = Column(String(20))
    answer_19 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        init_str = "<user_id:{}".format(user_id)
        for i in range(1, 20):
            init_str += " answer_{}: {}".format(i, get_attr(self, 'answer_%d'%(i)))
        return init_str

# Define Post questionnaire table:
class PostQ(Base):
    # table name
    __tablename__ = 'postq'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    answer_2 = Column(String(20))
    answer_3 = Column(String(20))
    answer_4 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} answer_1:{1} answer_2:{2} answer_3:{3} answer_4:{4}>".format(self.user_id,
         self.answer_1, self.answer_2, self.answer_3, self.answer_4)

class PostQ_distrust(Base):
    # table name
    __tablename__ = 'PostQ_distrust'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(500))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} answer_1:{1}>".format(self.user_id, self.answer_1)

class PostQ_accuracy(Base):
    # table name
    __tablename__ = 'PostQ_accuracy'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} answer_1:{1}>".format(self.user_id, self.answer_1)

class PostQ_ham(Base):
    # table name
    __tablename__ = 'PostQ_ham'

    # table structure
    user_id = Column(String(30), primary_key=True)
    malignant_features = Column(String(500))
    benign_features = Column(String(500))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} malignant_features:{1}\n benign_features:{2}>".format(self.user_id,
         self.malignant_features, self.benign_features)

class PostQ_ham_2(Base):
    # table name
    __tablename__ = 'PostQ_ham_2'

    # table structure
    user_id = Column(String(30), primary_key=True)
    opinion_explanation = Column(String(500))
    reason_explanation = Column(String(500))
    opinion_analogy = Column(String(500))
    reason_analogy = Column(String(500))
    comments_analogy = Column(String(500))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} opinion_explanation:{1}\n reason_explanation:{2} opinion_analogy:{3} reason_analogy:{4} comments_analogy:{5}>".format(self.user_id,
         self.opinion_explanation, self.reason_explanation, self.opinion_analogy, self.reason_analogy, self.comments_analogy)

class PostQ_ham_3(Base):
    # table name
    __tablename__ = 'PostQ_ham_3'

    # table structure
    user_id = Column(String(30), primary_key=True)
    reason_decision = Column(String(1000))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} reason_decision:{1}>".format(self.user_id,
         self.reason_decision)

class PostQ_analogy(Base):
    # table name
    __tablename__ = 'PostQ_analogy'

    # table structure
    user_id = Column(String(30), primary_key=True)
    answer_1 = Column(String(20))
    answer_2 = Column(String(20))
    answer_3 = Column(String(500))
    answer_4 = Column(String(500))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} answer_1:{1} answer_2:{2} answer_3:{3} answer_4:{4}>".format(self.user_id,
         self.answer_1, self.answer_2, self.answer_3, self.answer_4)

class UserInfo(Base):
    # table name
    __tablename__ = 'userinfo'

    # table structure
    user_id = Column(String(30), primary_key=True)
    task_order_str = Column(String(200))
    reverse_flag_str = Column(String(200))
    user_group = Column(String(20))
    analogy_type = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} task_id:{1} non_analogy_answer:{2} analogy_answer:{3}>".format(self.user_id,
         self.task_order_str, self.user_group, self.analogy_type)

class UserBehavior(Base):
    # table name
    __tablename__ = 'userbehavior'

    # table structure
    user_id = Column(String(30), primary_key=True)
    event_desc = Column(String(200), primary_key=True)
    user_behavior = Column(String(200))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} event:{1} behavior:{2}>".format(self.user_id,
         self.event_desc, self.user_behavior)

# Define User-Task table:
class UserTask(Base):
    # table name
    __tablename__ = 'usertask'

    # table structure
    user_id = Column(String(30), primary_key=True)
    task_id = Column(String(30), primary_key=True)
    answer_type = Column(String(20), primary_key=True)
    choice = Column(String(20))
    created_time = Column(DateTime, default=datetime.datetime.utcnow)
    # non_analogy_answer = Column(String(20))
    # non_anlogy_time = Column(DateTime)
    # analogy_answer = Column(String(20))
    # analogy_time = Column(DateTime, default=datetime.datetime.utcnow)

    def __repr__(self):    # make it printable
        return "<id:{0} task_id:{1} non_analogy_answer:{2} analogy_answer:{3}>".format(self.user_id,
         self.task_id, self.non_analogy_answer, self.analogy_answer)

# # Define Task table:
# class Task(Base):
#     # table name
#     __tablename__ = 'task'

#     # table structure
#     task_id = Column(String(20), primary_key=True)
#     education = Column(String(20))
#     loan_label = Column(String(20))
#     analogy = Column(String(20))
#     prediction_acc = Column(String(20))
#     created_time = Column(DateTime, default=datetime.datetime.utcnow)

#     def __repr__(self):    # make it printable
#         return "<id:{0} task_id:{1} analogy:{2} prediction_acc:{3}>".format(self.taks_id, self.analogy, self.prediction_acc)

# class LoanPrediction(Base):
#     # table name
#     __tablename__ = 'loan'

#     # table structure
#     Loan_ID = Column(String(20), primary_key=True)
#     Education = Column(String(20))
#     ApplicantIncome = Column(String(20))
#     CoapplicantIncome = Column(String(20))
#     LoanAmount = Column(String(20))
#     Loan_Amount_Term = Column(String(20))
#     Credit_History = Column(String(20))
#     Property_Area = Column(String(20))
#     Loan_Status = Column(String(20))


#     def __repr__(self):    # make it printable
#       return "<Loan_id:{0} education:{1} ApplicantIncome:{2} LoanAmount:{3} Loan_Amount_Term:{4}>".format(self.Loan_ID, self.education,
#        self.ApplicantIncome, self.LoanAmount, self.Loan_Amount_Term)

def create_all_tables():
    Base.metadata.create_all(engine)   # 创建表结构
    # if table exist, it will ignore this operation


def init_db():
    create_all_tables()
    get_db()


@click.command('init-db')
@with_appcontext
def init_db_command():
    """create new tables."""
    init_db()
    click.echo('Initialized the database.')


def init_app(app):
    app.teardown_appcontext(close_db)
    app.cli.add_command(init_db_command)
