import functools
import datetime
from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for, current_app
)

from flaskr.db import get_db, get_engine

bp = Blueprint('loan_analogy', __name__, url_prefix='/loan_analogy')


import numpy as np
import pandas as pd
def sample_task_for_user(total_task):
    return np.random.randint(total_task)

from flask_wtf import FlaskForm
from wtforms import StringField, RadioField, SubmitField
from wtforms.validators import DataRequired, Length
from sqlalchemy.sql import text
from sqlalchemy import update

from flask import jsonify
from flaskr.questionnaire import ATIForm, PreFrom
from flaskr.db import PreQ, ATI_PreQ, TiA_PostQ, Analogy_PreQ
vaccine_analogy = "The system is <strong>75%</strong> accurate, which is as reliable as <strong>the AstraZeneca vaccine to protect against covid</strong>"
train_analogy = "The system is <strong>75%</strong> accurate, which is about as reliable as <strong>the French trains are on punctuality</strong>"
weather_analogy = "The system is <strong>75%</strong> accurate, which is about as accurate as <strong>the seven day weather forecast</strong>"
# example_ID_list = ['LP001518', 'LP002723']

task_ID_list = ['LP002571', 'LP001633', 'LP001882', 'LP002181', 'LP001469', 'LP002500', 'LP002449', 'LP001722', 'LP002534', 'LP001451', 'LP001536', 'LP001996']
task_ID_mapping = {
    'LP001518': 'Jdk12380',
    'LP002723': 'GbF02909',
    'LP002571': 'gzx12398',
    'LP001633': 'obS23678',
    'LP001882': 'Psq98134',
    'LP002181': 'GSn12309',
    'LP001469': 'PtD23957',
    'LP002500': 'QJn78681',
    'LP002449': 'NSG63930',
    'LP001722': 'RSq33233',
    'LP002534': 'OoU19287',
    'LP001451': 'BBQ98172',
    'LP001536': 'LjW96235',
    'LP001996': 'WjZ12687',
}

# task2analogy_type = {
#     'LP001518': 'weather',
#     'LP002723': 'weather',
#     'LP002571': 'weather',
#     'LP001633': 'weather',
#     'LP001882': 'weather',
#     'LP002181': 'weather',
#     'LP001469': 'train',
#     'LP002500': 'train',
#     'LP002449': 'train',
#     'LP001722': 'train',
#     'LP002534': 'vaccine',
#     'LP001451': 'vaccine',
#     'LP001536': 'vaccine',
#     'LP001996': 'vaccine',
# }

# weather_tasks = ['LP002571', 'LP001633', 'LP001882', 'LP002181']
# train_tasks = ['LP001469', 'LP002500', 'LP002449', 'LP001722']
# vaccine_tasks = ['LP002534', 'LP001451', 'LP001536', 'LP001996']
task_list_1 = ['LP002571', 'LP001633', 'LP001882', 'LP002181']
task_list_2 = ['LP001469', 'LP002500', 'LP002449', 'LP001722']
task_list_3 = ['LP002534', 'LP001451', 'LP001536', 'LP001996']

class LoanData(object):

    def __init__(self, loan_task):
        Loan_ID,Gender,Married,Dependents,Education,Self_Employed,ApplicantIncome,CoapplicantIncome, \
        LoanAmount,Loan_Amount_Term,Credit_History,Property_Area,Loan_Status,Prediction_Score,Rank_By_Score,Sample_Category = loan_task
        self.Loan_ID = Loan_ID
        self.rand_ID = task_ID_mapping[Loan_ID]
        # self.analogy_type = task2analogy_type[Loan_ID]
        self.Gender = Gender
        self.Married = Married
        self.Dependents = str(Dependents)
        self.Self_Employed = Self_Employed
        self.Education = Education
        self.ApplicantIncome = self.deal_int(ApplicantIncome)
        self.CoapplicantIncome = self.deal_int(CoapplicantIncome)
        self.LoanAmount = self.deal_int(LoanAmount)
        self.Loan_Amount_Term = self.deal_int(Loan_Amount_Term)
        if Credit_History is not None:
            if Credit_History == 1.0:
                self.Credit_History = "Yes"
            else:
                self.Credit_History = "No"
        else:
            self.Credit_History = "NA"
        self.Property_Area = Property_Area
        self.Loan_Status = Loan_Status
        self.Prediction_Score = float(Prediction_Score)

    @staticmethod
    def deal_int(value):
        if value is not None:
            out_str = "%d"%(value)
        else:
            out_str = "0"
        return out_str

    def print_loan_task(self):
        # loan_str = "Statistics for loan applicant: "
        loan_str = ""
        loan_str += "Loan Applicant %s, %s"%(self.rand_ID, self.Gender.lower())
        loan_str += ", {}".format("married" if self.Married == "Yes" else "single")
        loan_str += ", with %s dependent(s)"%(self.Dependents)
        if self.Self_Employed == "Yes":
            loan_str += ", self employed."
        else:
            loan_str += "."
        loan_str += " {} has{} graduated from college.".format("He" if self.Gender == "Male" else "She", "n't" if 'Not' in self.Education else "")
        # if 'Not' in self.Education:
        #   loan_str += " {} has{} graduated from college".format("He" if self.Gender == "Male" else "She")
        # else:
        #   loan_str += " He (She) has graduated from college"
        loan_str += " Applicant's income is %s dollars per month"%(self.ApplicantIncome)
        loan_str += ", coapplicant's income is %s dollars per month"%(self.CoapplicantIncome)
        # if self.LoanAmount is not None:
        loan_str += ". %s loan amount is %s thousand dollars in total"%("His" if self.Gender == "Male" else "Her", self.LoanAmount)
        # if self.Loan_Amount_Term is not None:
        loan_str += ", and loan term is %s months"%(self.Loan_Amount_Term)
        loan_str += ". {} {} credit history".format("He" if self.Gender == "Male" else "She", "has" if self.Credit_History == "Yes" else "doesn't have")
        # if self.Credit_History == "Yes":
        #   loan_str += ". He (She) has credit history"
        # else:
        #   loan_str += ". He (She) doesn't have credit history"
        # if self.Property_Area is not None:
        loan_str += ", and possesses property at %s area."%(self.Property_Area)
        # loan_str += " Do you think we should lend money to {}?".format("him" if self.Gender == "Male" else "her")
        return loan_str

    def serize(self):
        # return jsonify(Loan_ID=self.Loan_ID, Education=self.Education, ApplicantIncome=self.ApplicantIncome,
        #  CoapplicantIncome=self.CoapplicantIncome, LoanAmount=self.LoanAmount, Loan_Amount_Term=self.Loan_Amount_Term,
        #  Credit_History=self.Credit_History, Property_Area=self.Property_Area, Loan_Status=self.Loan_Status)
        # Use rand_ID to show, to avoid users access to real Loan_ID
        return {
            "Loan_ID": self.rand_ID,
            "Gender": self.Gender,
            "Married": self.Married,
            "Dependents": self.Dependents,
            "Self_Employed": self.Self_Employed,
            "Education": self.Education,
            "ApplicantIncome": self.ApplicantIncome,
            "CoapplicantIncome": self.CoapplicantIncome,
            "LoanAmount": self.LoanAmount + " k",
            "Loan_Amount_Term": self.Loan_Amount_Term,
            "Credit_History": self.Credit_History,
            "Property_Area": self.Property_Area,
            "Prediction_Score": self.Prediction_Score,
            "Loan_Status": self.Loan_Status
        }

def load_loan_data(data_file="flaskr/static/loan_data/boarderline_12_tasks.csv"):
    df = pd.read_csv(data_file)
    data_dict = {}
    for i in range(len(df)):
        tp_data = LoanData(df.iloc[i,:].tolist())
        data_dict[tp_data.Loan_ID] = tp_data
    # print(data_dict['LP001518'].print_loan_task())
    return data_dict

# Loading task information from local file
loan_task_list = load_loan_data()

def get_loan_data(task_id = 'LP002571'):
    return loan_task_list[task_id].serize()

class submit_form(FlaskForm):
    submit_button = SubmitField('Got it! Next!')

class attention_form(FlaskForm):
    loan_choice = RadioField('Is 77 divisible by 11?',
     choices=[('yes','yes'),('no','no')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class attention_form_2(FlaskForm):
    loan_choice = RadioField('Is 3 + 5 = 9?',
     choices=[('yes','yes'),('no','no')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class loan_form(FlaskForm):
    loan_choice = RadioField('Do you decide to accept or reject this application?',
     choices=[('accept','accept'),('reject','reject')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class loan_form_advice(FlaskForm):
    loan_choice = RadioField("Considering the system's advice, do you decide to accept or reject this application?",
     choices=[('accept','accept'),('reject','reject')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')


Likert_6_choices = choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')]
class AnalogyForm(FlaskForm):
    answer_1 = RadioField('How familiar are you with punctuality of trains?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_2 = RadioField('How familiar are you with weather forecasts?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_3 = RadioField('How familiar are you with COVID-19 Vaccine?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_4 = RadioField('To what extent do you trust the train punctuality?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_5 = RadioField('To what extent do you trust the weather forecast?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_6 = RadioField('To what extent do you trust the the effectiveness of COVID-19 vaccine?', choices=Likert_6_choices, validators=[DataRequired()])
    
    answer_7 = RadioField('How confident are you with estimating the punctuality of trains numerically?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_8 = RadioField('How confident are you with estimating the accuracy of weather forecasts numerically?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_9 = RadioField('How confident are you with estimating the effectiveness of vaccines numerically?', choices=Likert_6_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

class AnalogyTrainForm(FlaskForm):
    answer_1 = RadioField('How familiar are you with punctuality of French trains?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_2 = RadioField('To what extent do you trust the French train punctuality?', choices=Likert_6_choices, validators=[DataRequired()])  
    answer_3 = RadioField('How confident are you with estimating the punctuality of French trains numerically?', choices=Likert_6_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

class AnalogyWeatherForm(FlaskForm):
    answer_1 = RadioField('How familiar are you with seven day weather forecasts?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_2 = RadioField('To what extent do you trust the seven day weather forecast?', choices=Likert_6_choices, validators=[DataRequired()])  
    answer_3 = RadioField('How confident are you with estimating the accuracy of seven day weather forecasts numerically?', choices=Likert_6_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

class AnalogyVaccineForm(FlaskForm):
    answer_1 = RadioField('How familiar are you with AstraZeneca vaccine for COVID-19?', choices=Likert_6_choices, validators=[DataRequired()])
    answer_2 = RadioField('To what extent do you trust the effectiveness of AstraZeneca vaccine for COVID-19?', choices=Likert_6_choices, validators=[DataRequired()])  
    answer_3 = RadioField('How confident are you with estimating the effectiveness of AstraZeneca vaccine for COVID-19 numerically?', choices=Likert_6_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

TiA_choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')]
class TiA_Form(FlaskForm):

    # Familiarity
    answer_1 = RadioField('I already know similar systems.', choices=TiA_choices, validators=[DataRequired()])
    answer_2 = RadioField('I have already used similar systems.', choices=TiA_choices, validators=[DataRequired()])

    # Attention check
    answer_6 = RadioField('Attention check: choose option strongly disagree.', choices=TiA_choices, validators=[DataRequired()])

    # Propensity to Trust
    answer_3 = RadioField('One should be careful with unfamiliar automated systems.', choices=TiA_choices, validators=[DataRequired()])
    answer_4 = RadioField('I rather trust a system than I mistrust it.', choices=TiA_choices, validators=[DataRequired()])
    answer_5 = RadioField('Automated systems generally work well.', choices=TiA_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')


def save_user_info(user_id, user_task_order, reverse_flag_list, analogy_domain_order, user_group, analogy_type):
    from flaskr.db import UserInfo
    db = get_db()
    q = db.query(UserInfo.user_id).filter(UserInfo.user_id==user_id)
    exist_flag = db.query(q.exists()).scalar()
    # if exist_flag:
    #     return True
    task_order_str = "|".join(user_task_order)
    reverse_flag_str = "|".join(reverse_flag_list)
    analogy_domain_str = "|".join(analogy_domain_order)
    task_order_str = task_order_str + " " + analogy_domain_str

    if not exist_flag:
        ui_obj = UserInfo(user_id=user_id, task_order_str=task_order_str, reverse_flag_str=reverse_flag_str, user_group=user_group, analogy_type=analogy_type)
        db.add(ui_obj)
        db.commit()
    else:
        stmt = update(UserInfo).where(UserInfo.user_id==user_id).values(task_order_str=task_order_str, reverse_flag_str=reverse_flag_str, user_group=user_group, analogy_type=analogy_type).execution_options(synchronize_session="fetch")
        result = db.execute(stmt)
        db.commit()
        current_app.logger.info("User {} exist, update userinfo".format(user_id))

    return False

@bp.route('/', methods=['GET', 'POST'])
def user_port():
    user_id = request.args.get('PROLIFIC_PID')
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0

    tp_task_ID_list = np.random.permutation(task_list_1).tolist() + np.random.permutation(task_list_2).tolist() + np.random.permutation(task_list_3).tolist()
    session['task_ID_list'] = tp_task_ID_list
    reverse_task_set = set()
    # randomly select one task from each group (4 tasks) to provide wrong advice
    reverse_task_set.add(np.random.choice(task_list_1, 1)[0])
    reverse_task_set.add(np.random.choice(task_list_2, 1)[0])
    reverse_task_set.add(np.random.choice(task_list_3, 1)[0])
    assert len(reverse_task_set) == 3
    # reverse_flag_list = np.random.permutation(['True'] * 3 + ['False'] * 9).tolist()
    # True means reverse the advice from correct.
    # randomly assign analogy domains to three groups of tasks
    analogy_domain_order = np.random.permutation(['weather', 'train', 'vaccine']).tolist()
    task2analogy_type = {}
    for task_id in task_list_1:
        task2analogy_type[task_id] = analogy_domain_order[0]
    for task_id in task_list_2:
        task2analogy_type[task_id] = analogy_domain_order[1]
    for task_id in task_list_3:
        task2analogy_type[task_id] = analogy_domain_order[2]
    reverse_flag_list = []
    system_advice_list = []
    tp_task_analogy_list = []
    for i, task_id in enumerate(tp_task_ID_list):
        tp_task_analogy_list.append(task2analogy_type[task_id])
        loan_data = get_loan_data(task_id)
        if task_id in reverse_task_set:
            reverse_flag = 'True'
        else:
            reverse_flag = 'False'
        reverse_flag_list.append(reverse_flag)
        if eval(reverse_flag):
            # reverse the answer to show
            system_advice = "reject" if loan_data['Loan_Status'] == "Y" else "accept"
        else:
            system_advice = "accept" if loan_data['Loan_Status'] == "Y" else "reject"
        system_advice_list.append(system_advice)
    session['system_advice_list'] = system_advice_list
    session['reverse_flag_list'] = reverse_flag_list
    session['task_analogy_list'] = tp_task_analogy_list
    session['analogy_domain_order'] = analogy_domain_order

    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, reverse_flag_list=reverse_flag_list,
        analogy_domain_order=analogy_domain_order, user_group="loan_analogy", analogy_type="all")
    # if exist_flag:
    #     return render_template('thanks.html', reason_str = "You have participated this task once. We don't permit one person for multiple participation")
    #     app.logger.info("User {} come in for more than one time, show him the thanks page".format(user_id))
    session['check_pass'] = False
    session['check_pass_2'] = False

    current_app.logger.info("User {} log in".format(user_id))
    return redirect(url_for("loan_analogy.instruction"))

@bp.route('/instruction', methods=('GET', 'POST'))
def instruction():
    # show the user instruction for whole task
    form = submit_form()
    if request.method == 'POST':
        return redirect(url_for("loan_analogy.preq"))
    return render_template('loan_analogy/instruction.html', form=form)

@bp.route('/example_1', methods=('GET', 'POST'))
def example_1():
    form = submit_form()
    if request.method == 'POST':
        session['non_anlogy_time'] = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        return redirect(url_for('loan_analogy.example_2'))
    else:
        loan_str, loan_data = fetch_task_info(task_id= 'LP001518')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan_analogy/example.html', task_index=1, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/example_2', methods=('GET', 'POST'))
def example_2():
    form = submit_form()
    if request.method == 'POST':
        leave_time = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        # return redirect(url_for('loan_analogy.base'))
        return redirect(url_for('loan_analogy.show_analogy'))
    else:
        enter_time = datetime.datetime.utcnow()
        loan_str, loan_data = fetch_task_info(task_id= 'LP002723')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan_analogy/example.html', task_index=2, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/show_analogy', methods=('GET', 'POST'))
def show_analogy():
    task_id = session['task_id']
    task_analogy_list = session['task_analogy_list']
    analogy_type = task_analogy_list[task_id]
    if analogy_type == "vaccine":
        analogy_str = vaccine_analogy
        form = AnalogyVaccineForm()
    elif analogy_type == "train":
        analogy_str = train_analogy
        form = AnalogyTrainForm()
    else:
        analogy_str = weather_analogy
        form = AnalogyWeatherForm()
    if request.method == 'POST':
        # in this step, we only record user's decision in non-analogy case
        if form.validate_on_submit():
            user_id = session['user_id']
            base_index = [1, 4, 7]
            shift = {"train": 0, "weather": 1, "vaccine": 2}
            answer_1 = form.answer_1.data
            answer_2 = form.answer_2.data
            answer_3 = form.answer_3.data
            answer_dict = {"user_id": user_id}
            for i in range(3):
                tp_key = 'answer_{}'.format(i + 1)
                ans_key = 'answer_{}'.format(base_index[i] + shift[analogy_type])
                answer_dict[ans_key] = getattr(form, tp_key).data
            db = get_db()
            error = None
            q = db.query(Analogy_PreQ.user_id).filter(Analogy_PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                preq_obj = Analogy_PreQ(**answer_dict)
                current_app.logger.info("User {}'s data is recorded into Analogy_PreQ".format(user_id))
                db.add(preq_obj)
                db.commit()
            else:
                stmt = update(Analogy_PreQ).where(Analogy_PreQ.user_id==user_id).values(**answer_dict).execution_options(synchronize_session="fetch")
                result = db.execute(stmt)
                db.commit()
                current_app.logger.info("User {} Analogy_preq exist, update Analogy_preq".format(user_id))
            return redirect(url_for('loan_analogy.base'))
    response = render_template('loan_analogy/switch.html', analogy_str=analogy_str, form=form)
    return response
    # response.set_cookie('task_id', task_id)
    # return response

def save_choice(user_id, task_id, choice, answer_type="attention"):
    db = get_db()
    error = None
    from flaskr.db import UserTask
    q = db.query(UserTask.user_id).filter(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = UserTask(user_id=user_id, task_id=task_id, choice=choice, answer_type=answer_type)
        db.add(ut_obj)
        db.commit()
    else:
        stmt = update(UserTask).where(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type).values(choice=choice).execution_options(synchronize_session="fetch")
        result = db.execute(stmt)
        db.commit()
        current_app.logger.info("User {} has record taskid-answer_type:{}-{} exist, update usertask".format(user_id, task_id, answer_type))

@bp.route('/preq', methods=('GET', 'POST'))
def preq():
    form = PreFrom()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in pre-questionnaire".format(user_id))
            # answer_1 = request.form['answer_1']
            # answer_2 = request.form['answer_2']
            # answer_3 = request.form['answer_3']
            # answer_4 = request.form['answer_4']
            answer_1 = form.answer_1.data
            answer_2 = form.answer_2.data
            answer_3 = form.answer_3.data
            answer_4 = form.answer_4.data
            answer_5 = form.answer_5.data
            answer_6 = form.answer_6.data
            answer_7 = form.answer_7.data
            answer_8 = form.answer_8.data
            tp_choice = form.answer_9.data
            save_choice(user_id, task_id="Numeracy Test", choice=tp_choice, answer_type="attention")
            if tp_choice == '3':
                session['attention_chck_right'] += 1
            else:
                session['attention_chck_fail'] += 1
                if session['attention_chck_fail'] >= 2:
                    return redirect("https://app.prolific.co/submissions/complete?cc=257F8CFA")
            db = get_db()
            error = None
            q = db.query(PreQ.user_id).filter(PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                # print(user_id)
                preq_obj = PreQ(user_id=user_id, answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4,\
                 answer_5=answer_5, answer_6=answer_6, answer_7=answer_7, answer_8=answer_8)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            else:
                stmt = update(PreQ).where(PreQ.user_id==user_id).values(answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4,\
                 answer_5=answer_5, answer_6=answer_6, answer_7=answer_7, answer_8=answer_8).execution_options(synchronize_session="fetch")
                result = db.execute(stmt)
                db.commit()
                current_app.logger.info("User {} preq exist, update preq".format(user_id))
            current_app.logger.info("User {}'s data is saved into database, then move to ATI questionnaire".format(user_id))
            return redirect(url_for('loan_analogy.ATI_preq'))

            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/pre.html', form=form)

@bp.route('/ATI_preq', methods=('GET', 'POST'))
def ATI_preq():
    form = ATIForm()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in pre-questionnaire".format(user_id))
            # answer_1 = form.answer_1.data
            # answer_2 = form.answer_2.data
            # answer_3 = form.answer_3.data
            # answer_4 = form.answer_4.data
            # answer_5 = form.answer_5.data
            # answer_6 = form.answer_6.data
            # answer_7 = form.answer_7.data
            # answer_8 = form.answer_8.data
            # answer_9 = form.answer_9.data
            answer_dict = {"user_id": user_id}
            for i in range(9):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            tp_choice = form.answer_10.data
            save_choice(user_id, task_id="ATI_attention", choice=tp_choice, answer_type="attention")
            if tp_choice == '6':
                session['attention_chck_right'] += 1
            else:
                session['attention_chck_fail'] += 1
                if session['attention_chck_fail'] >= 2:
                    return redirect("https://app.prolific.co/submissions/complete?cc=257F8CFA")
            db = get_db()
            error = None
            # from flaskr.db import ATI_PreQ
            # print(user_id)
            # preq_obj = ATI_PreQ(user_id=user_id, answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4,\
            #  answer_5=answer_5, answer_6=answer_6, answer_7=answer_7, answer_8=answer_8, answer_9=answer_9)
            q = db.query(ATI_PreQ.user_id).filter(ATI_PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                preq_obj = ATI_PreQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            else:
                stmt = update(ATI_PreQ).where(ATI_PreQ.user_id==user_id).values(**answer_dict).execution_options(synchronize_session="fetch")
                result = db.execute(stmt)
                db.commit()
                current_app.logger.info("User {} ATI_preq exist, update ATI_PreQ".format(user_id))
            # current_app.logger.info("User {}'s data is saved into database, then move to the Analogy questionnaire".format(user_id))
            current_app.logger.info("User {}'s data is saved into database, then move to the first example".format(user_id))
            # return redirect(url_for('loan_analogy.Analogy_preq'))
            return redirect(url_for('loan_analogy.example_1'))


            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/ATI_pre.html', form=form)

@bp.route('/Analogy_preq', methods=('GET', 'POST'))
def Analogy_preq():
    form = AnalogyForm()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in pre-questionnaire".format(user_id))
            answer_dict = {"user_id": user_id}
            for i in range(9):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            db = get_db()
            error = None
            q = db.query(Analogy_PreQ.user_id).filter(Analogy_PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                preq_obj = Analogy_PreQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            else:
                stmt = update(Analogy_PreQ).where(Analogy_PreQ.user_id==user_id).values(**answer_dict).execution_options(synchronize_session="fetch")
                result = db.execute(stmt)
                db.commit()
                current_app.logger.info("User {} Analogy_preq exist, update Analogy_preq".format(user_id))
            current_app.logger.info("User {}'s data is saved into database, then move to the first example".format(user_id))
            return redirect(url_for('loan_analogy.example_1'))

            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('loan_analogy/analogy_pre.html', form=form)

def fetch_task_info(task_id = 'LP001002'):
    task_item = loan_task_list[task_id]
    loan_data = task_item.serize()
    loan_str = task_item.print_loan_task()
    return loan_str, loan_data

@bp.route('/attention_check', methods=('GET', 'POST'))
def attention_check():
    form = attention_form()
    if request.method == 'POST':
        session['check_pass'] = True
        user_id = session['user_id']
        choice_tp = form.loan_choice.data
        if choice_tp == "yes":
            session['attention_chck_right'] += 1
        else:
            session['attention_chck_fail'] += 1
            if session['attention_chck_fail'] >= 2:
                return redirect("https://app.prolific.co/submissions/complete?cc=257F8CFA")
        save_choice(user_id, 'LP001518', choice_tp, answer_type="attention")
        leave_time = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        return redirect(url_for('loan_analogy.show_analogy'))
        # return redirect(url_for('loan_analogy.base'))
    else:
        enter_time = datetime.datetime.utcnow()
        loan_str, loan_data = fetch_task_info(task_id= 'LP001518')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan_analogy/attention_check.html', form=form, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/attention_check_2', methods=('GET', 'POST'))
def attention_check_2():
    form = attention_form_2()
    if request.method == 'POST':
        session['check_pass_2'] = True
        choice_tp = form.loan_choice.data
        user_id = session['user_id']
        if choice_tp == "no":
            session['attention_chck_right'] += 1
        else:
            session['attention_chck_fail'] += 1
            if session['attention_chck_fail'] >= 2:
                return redirect("https://app.prolific.co/submissions/complete?cc=257F8CFA")
        save_choice(user_id, 'LP002723', choice_tp, answer_type="attention")
        # leave_time = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        # return redirect(url_for('loan_analogy.base'))
        return redirect(url_for('loan_analogy.show_analogy'))
    else:
        enter_time = datetime.datetime.utcnow()
        loan_str, loan_data = fetch_task_info(task_id= 'LP002723')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan_analogy/attention_check.html', form=form, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/TiA_postq', methods=('GET', 'POST'))
def TiA_postq():
    form = TiA_Form()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in new TiA post-questionnaire".format(user_id))
            answer_dict = {"user_id": user_id}
            for i in range(5):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            tp_choice = form.answer_6.data
            save_choice(user_id, task_id="TiA_attention", choice=tp_choice, answer_type="attention")
            if tp_choice == '1':
                session['attention_chck_right'] += 1
            else:
                session['attention_chck_fail'] += 1
                if session['attention_chck_fail'] >= 2:
                    return redirect("https://app.prolific.co/submissions/complete?cc=257F8CFA")
            db = get_db()
            error = None
            q = db.query(TiA_PostQ.user_id).filter(TiA_PostQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                # print(user_id)
                preq_obj = TiA_PostQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            else:
                stmt = update(TiA_PostQ).where(TiA_PostQ.user_id==user_id).values(**answer_dict).execution_options(synchronize_session="fetch")
                result = db.execute(stmt)
                db.commit()
                current_app.logger.info("User {} TiA_PostQ exist, update TiA_PostQ".format(user_id))
            current_app.logger.info("User {}'s data is saved into database, then move to ATI questionnaire".format(user_id))
            return redirect(url_for('loan_analogy.postq'))

            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('loan_analogy/TiA_post_new.html', form=form)

@bp.route('/postq', methods=('GET', 'POST'))
def postq():
    # to-do: different group of participants may see different questionnaires
    # if session['attention_chck_right'] > 0:
    #     at least one attention check is passed
    #     return redirect("https://app.prolific.co/submissions/complete?cc=61FE0119")
    # if session['attention_chck_right'] < 3:
    if session['attention_chck_fail'] >= 2:
        # fail at more than 2 attention checks
        # return render_template('thanks.html', reason_str="Unfortunately, you have failed at least two attention checks and will not be allowed to proceed any further.")
        return redirect("https://app.prolific.co/submissions/complete?cc=257F8CFA")
    else:
        # return render_template('thanks.html', reason_str = "Congratulations! You have finished it!")
        # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
        return redirect("https://app.prolific.co/submissions/complete?cc=55C88AB7")

@bp.route('/base', methods=('GET', 'POST'))
def base():
    form = loan_form()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        choice_tp = form.loan_choice.data
        save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="base")
        return redirect(url_for('loan_analogy.analogy'))
    else:
        assert task_id < len(task_ID_list)
        # get Loan_ID to select a data sample to show
        session['last_base_task'] = task_ID_list[task_id]
        loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    # To check further, whether we need attention check here
    if task_id == 4 and not session['check_pass']:
        # At task 5, come to attention check
        response = redirect(url_for('loan_analogy.attention_check'))
        return response
    if task_id == 8 and not session['check_pass_2']:
        # At task 9, come to attention check 2
        response = redirect(url_for('loan_analogy.attention_check_2'))
        return response
    response = render_template('loan_analogy/base.html', form=form, loan_str=loan_str, loan_data=loan_data, task_index=task_id+1, total_task=len(task_ID_list))
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/analogy', methods=['GET', 'POST'])
def analogy():
    form = loan_form_advice()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    task_analogy_list = session['task_analogy_list']
    tp_task_ID = task_ID_list[task_id]
    analogy_type = task_analogy_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        choice_tp = form.loan_choice.data
        save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="loan_analogy")
        if task_id < len(task_ID_list) -1:
            # inser before redirection to next page
            session['task_id'] += 1
            # start next task
            return redirect(url_for('loan_analogy.base'))
        else:
            return redirect(url_for('loan_analogy.TiA_postq'))
        # return redirect(url_for("analogy_generation.analogy_evaluation"))
    else:
        assert task_id < len(task_ID_list)
        assert session['last_base_task'] == task_ID_list[task_id]
        loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
    if analogy_type == "vaccine":
        analogy_str = vaccine_analogy
    elif analogy_type == "train":
        analogy_str = train_analogy
    else:
        analogy_str = weather_analogy
    system_advice = session['system_advice_list'][task_id]
    analogy_str = "{}, and it choose to <strong>{}</strong> this application".format(analogy_str, system_advice)
    response = render_template('loan_analogy/analogy.html', form=form, loan_str=loan_str, \
     loan_data=loan_data, task_index=task_id+1, analogy_str=analogy_str, total_task=len(task_ID_list))
    return response


