import functools
import datetime
from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for
)
from werkzeug.security import check_password_hash, generate_password_hash

from flaskr.db import get_db, get_engine, PostQ_distrust

bp = Blueprint('loan_task', __name__, url_prefix='/loan_task')


import numpy as np
def sample_task_for_user(total_task):
    return np.random.randint(total_task)

from flask_wtf import FlaskForm
from wtforms import StringField, RadioField, SubmitField
from wtforms.validators import DataRequired, Length
from sqlalchemy.sql import text
class distrust_form(FlaskForm):
    answer_1 = StringField("Please explain why you switched your decision to oppose the system's advice.", validators=[DataRequired(), Length(max=500)])
    submit_button = SubmitField('Submit')

class submit_form(FlaskForm):
    submit_button = SubmitField('Got it! Next!')

class attention_form(FlaskForm):
    loan_choice = RadioField('Is 77 divisible by 11?',
     choices=[('yes','yes'),('no','no')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class attention_form_2(FlaskForm):
    loan_choice = RadioField('Is 3 + 5 = 9?',
     choices=[('yes','yes'),('no','no')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class loan_form(FlaskForm):
    loan_choice = RadioField('Do you decide to accept or reject this application?',
     choices=[('accept','accept'),('reject','reject')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class loan_form_advice(FlaskForm):
    loan_choice = RadioField("Considering the system's advice, do you decide to accept or reject this application?",
     choices=[('accept','accept'),('reject','reject')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

from flask import jsonify
from flaskr.util import load_loan_data, vaccine_analogy, train_analogy, weather_analogy
# Loading task information from local file
loan_task_list = load_loan_data()

def obtain_user_choices(user_id, task_ID_list, user_group):
    engine = get_engine()
    cmd = 'SELECT choice FROM usertask WHERE user_id = :user_id and task_id = :task_id and answer_type = :answer_type'
    first_choice_list = []
    second_choice_list = []
    with engine.connect() as con:
        for task_id in task_ID_list:
            user_choices_1 = con.execute(text(cmd), user_id=user_id, task_id=task_id, answer_type="base").fetchone()
            user_choices_2 = con.execute(text(cmd), user_id=user_id, task_id=task_id, answer_type=user_group).fetchone()
            first_choice_list.append(user_choices_1[0])
            second_choice_list.append(user_choices_2[0])
        return first_choice_list, second_choice_list



def check_user_finish(user_id, user_group, task_ID_list):
    engine = get_engine()
    cmd = 'SELECT task_id, answer_type FROM usertask WHERE user_id = :user_id'
    assert user_group in ["base", "system", "accuracy", "analogy"]
    # required_tasks = [('LP001518', 'attention'), ('LP002723', 'attention')]
    required_tasks = [('LP001518', 'attention'), ('ATI_attention', 'attention'), ('', '')]
    for task_id in task_ID_list:
        required_tasks.append((task_id, "base"))
        if user_group != "base":
            required_tasks.append((task_id, user_group))
    with engine.connect() as con:
        user_choices = con.execute(text(cmd), user_id=user_id).fetchall()
        print(type(user_choices))
        print(user_choices)
        print(len(required_tasks), len(user_choices))
        if user_group == "base":
            if len(user_choices) == 13:
                # attention check and 10 tasks
                return True, set()
            else:
                return False, set(required_tasks) - set(user_choices)
        else:
            if len(user_choices) == 23:
                return True, set()
            else:
                return False, set(required_tasks) - set(user_choices)

def fetch_task_info(task_id = 'LP001002'):
    # engine = get_engine()
    # cmd = 'SELECT * FROM loan WHERE Loan_ID = :task_id'
    task_item = loan_task_list[task_id]
    loan_data = task_item.serize()
    loan_str = task_item.print_loan_task()
    # with engine.connect() as con:
    #     loan_task = con.execute(text(cmd), task_id=task_id).fetchone()
    #     # Loan_ID, Education, ApplicantIncome, CoapplicantIncome, LoanAmount,
    #     # Loan_Amount_Term, Credit_History, Property_Area ,Loan_Status = loan_task
    #     loan_str = print_loan_task(loan_task)
    #     loan_data = LoanData(loan_task).serize()
    return loan_str, loan_data

@bp.route('/example_1', methods=('GET', 'POST'))
def example_1():
    form = submit_form()
    if request.method == 'POST':
        session['non_anlogy_time'] = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        return redirect(url_for('loan_task.example_2'))
    else:
        loan_str, loan_data = fetch_task_info(task_id= 'LP001518')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan/example.html', task_index=1, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/example_2', methods=('GET', 'POST'))
def example_2():
    form = submit_form()
    if request.method == 'POST':
        leave_time = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        return redirect(url_for('loan_task.base'))
    else:
        enter_time = datetime.datetime.utcnow()
        loan_str, loan_data = fetch_task_info(task_id= 'LP002723')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan/example.html', task_index=2, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/attention_check', methods=('GET', 'POST'))
def attention_check():
    form = attention_form()
    if request.method == 'POST':
        session['check_pass'] = True
        user_id = session['user_id']
        choice_tp = form.loan_choice.data
        if choice_tp == "yes":
            session['attention_chck_right'] += 1
        save_choice(user_id, 'LP001518', choice_tp, answer_type="attention")
        leave_time = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        return redirect(url_for('loan_task.base'))
    else:
        enter_time = datetime.datetime.utcnow()
        loan_str, loan_data = fetch_task_info(task_id= 'LP001518')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan/attention_check.html', form=form, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

@bp.route('/attention_check_2', methods=('GET', 'POST'))
def attention_check_2():
    form = attention_form_2()
    if request.method == 'POST':
        session['check_pass'] = True
        choice_tp = form.loan_choice.data
        user_id = session['user_id']
        if choice_tp == "no":
            session['attention_chck_right'] += 1
        save_choice(user_id, 'LP002723', choice_tp, answer_type="attention")
        # leave_time = datetime.datetime.utcnow()
        # in this step, we only record user's decision in non-analogy case
        return redirect(url_for('questionnaire.TiA_postq'))
    else:
        enter_time = datetime.datetime.utcnow()
        loan_str, loan_data = fetch_task_info(task_id= 'LP002723')
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    response = render_template('loan/attention_check.html', form=form, loan_str=loan_str, loan_data=loan_data)
    # response.set_cookie('task_id', task_id)
    return response

def generate_report(first_choice_list, second_choice_list, system_advice_list, reverse_flag_list):
    assert len(first_choice_list) == 10 and len(second_choice_list) == 10
    # print(len(system_advice_list), len(reverse_flag_list))
    assert len(system_advice_list) == 10 and len(reverse_flag_list) == 10
    correct_answer_list = []
    response_str_list = []
    strange_task_list = []
    total_correct = 0
    for i in range(10):
        reverse_flag = eval(reverse_flag_list[i])
        response_str = ""
        if first_choice_list[i] != second_choice_list[i]:
            if second_choice_list[i] == system_advice_list[i]:
                # response_str += "You adopted system's advice to change your decision."
                response_str += "You adopted the system's advice and changed your initial decision."
                strange_task_list.append(False)
            else:
                # response_str += "You distrust system advice and changed your decision to the opposite."
                response_str += "You distrusted the system’s advice and changed your decision to the opposite choice."
                strange_task_list.append(True)
        else:
            strange_task_list.append(False)
            if first_choice_list[i] == system_advice_list[i]:
                # response_str += "You kept your decision, which is same with system advice."
                response_str += "You did not change your decision, which was consistent with the system’s advice."
            else:
                # response_str += "You kept your decision, which is different with system advice."
                response_str += "You did not change your decision, which was different from the system’s advice."
        if second_choice_list[i] == system_advice_list[i]:
            if reverse_flag:
                # correct answer is reverse from system advice
                correct_answer_list.append("accept" if system_advice_list[i] == "reject" else "reject")
                response_str += " And it turns out to be wrong."
            else:
                correct_answer_list.append(system_advice_list[i])
                response_str += " And it turns out to be correct."
                total_correct += 1
        else:
            if reverse_flag:
                # correct answer is reverse from system advice
                correct_answer_list.append("accept" if system_advice_list[i] == "reject" else "reject")
                response_str += " And it turns out to be correct."
                total_correct += 1
            else:
                correct_answer_list.append(system_advice_list[i])
                response_str += " And it turns out to be wrong."
        response_str_list.append(response_str)
    return correct_answer_list, response_str_list, strange_task_list, total_correct


@bp.route('/result', methods=('GET', 'POST'))
def result():
    system_advice_list = session['system_advice_list']
    task_ID_list = session['task_ID_list']
    user_id = session['user_id']
    reverse_flag_list = session['reverse_flag_list']
    user_group = session['user_group']
    first_choice_list, second_choice_list = obtain_user_choices(user_id=user_id, task_ID_list=task_ID_list, user_group=user_group)
    correct_answer_list, response_str_list, strange_task_list, total_correct = generate_report(first_choice_list, second_choice_list, system_advice_list, reverse_flag_list)
    strange_str = ""
    if any(strange_task_list):
        for i, flag in enumerate(strange_task_list):
            if flag:
                if len(strange_str) == 0:
                    strange_str += "On question {}".format(i + 1)
                else:
                    strange_str += ", {}".format(i + 1)
        strange_str += ", your first decision was the same as the system's advice, but after receiving the system's advice, you changed your decision."
        # strange_str += "."
        form = distrust_form()
    else:
        form = submit_form()
    if request.method == 'POST' and form.validate_on_submit():
        if any(strange_task_list):
            user_response = form.answer_1.data
            save_distrust_reason(user_id, user_response)
        if session['attention_chck_right'] >= 3:
            # at least one attention check is passed
            # return redirect("https://app.prolific.co/submissions/complete?cc=61FE0119")
            return redirect("https://app.prolific.co/submissions/complete?cc=55C88AB7")
            # return "Congratulations! You finished all tasks."
        else:
            # at least one attention check is passed
            return render_template('thanks.html', reason_str="Unfortunately, you have failed more than 2 attention checks and will not be allowed to proceed any further.")
    encourage_str = "Congratulations! You achieved {} correct decisions among 10 tasks.".format(total_correct)
    response = render_template('loan/result.html', form=form, user_choices=second_choice_list, response_str_list=response_str_list, encourage_str=encourage_str, strange_str=strange_str)
    # response.set_cookie('task_id', task_id)
    return response

def save_distrust_reason(user_id, user_response):
    db = get_db()
    error = None
    from flaskr.db import PostQ_distrust
    q = db.query(PostQ_distrust.user_id).filter(PostQ_distrust.user_id==user_id)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        pd_obj = PostQ_distrust(user_id=user_id, answer_1=user_response)
        db.add(pd_obj)
        db.commit()

def save_choice(user_id, task_id, choice, answer_type="non_analogy"):
    db = get_db()
    error = None
    from flaskr.db import UserTask
    q = db.query(UserTask.user_id).filter(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = UserTask(user_id=user_id, task_id=task_id, choice=choice, answer_type=answer_type)
        db.add(ut_obj)
        db.commit()

@bp.route('/base', methods=('GET', 'POST'))
def base():
    form = loan_form()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    if request.method == 'POST' and form.validate_on_submit():
        # user_id = request.cookies.get('user_id')
        user_id = session['user_id']
        # task_id = g.task
        # g.non_analogy_answer = request.form['answer_1']
        # g.non_analogy_answer = form.loan_choice
        # key_ = "base_choice_{}".format(task_id)
        choice_tp = form.loan_choice.data
        # session[key_] = choice_tp
        save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="base")
        user_group = session['user_group']
        if user_group == "system":
            # provide system's choice
            return redirect(url_for('loan_task.system'))
        elif user_group == "accuracy":
            # provide system's choice and 
            return redirect(url_for('loan_task.accuracy'))
        elif user_group == "analogy":
            # provide system's choice and analogy-based explanation
            return redirect(url_for('loan_task.analogy'))
        else:
            # basic group participants, do only basic choices
            if task_id < len(task_ID_list) -1:
                session['task_id'] += 1
                # start next task
                return redirect(url_for('loan_task.base'))
            else:
                # finished, reserved_set = check_user_finish(user_id=user_id, user_group="base", task_ID_list=task_ID_list)
                return redirect(url_for('loan_task.attention_check_2'))
                if finished:
                    # end of tasks come to 2nd attention check question
                    return redirect(url_for('loan_task.attention_check_2'))
                else:
                    # To-do, continue to fill in unfinished tasks
                    return redirect(url_for('loan_task.attention_check_2'))
    else:
        assert task_id < len(task_ID_list)
        # get Loan_ID to select a data sample to show
        session['last_base_task'] = task_ID_list[task_id]
        loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
        # use default task_id 'LP001002'
        # deliver pre-task questionnaire page
    if task_id == 5 and not session['check_pass']:
        # At task 6, come to attention check
        response = redirect(url_for('loan_task.attention_check'))
        return response
    response = render_template('loan/base.html', form=form, loan_str=loan_str, loan_data=loan_data, task_index=task_id+1)
    # response.set_cookie('task_id', task_id)
    return response

# @bp.route('/non_analogy', methods=('GET', 'POST'))
# def non_analogy():
#     form = loan_form()
#     task_id = session['task_id']
#     task_ID_list = session['task_ID_list']
#     if request.method == 'POST' and form.validate_on_submit():
#         # user_id = request.cookies.get('user_id')
#         user_id = session['user_id']
#         # task_id = g.task
#         # g.non_analogy_answer = request.form['answer_1']
#         # g.non_analogy_answer = form.loan_choice
#         key_ = "non_analogy_choice_{}".format(task_id)
#         choice_tp = form.loan_choice.data
#         session[key_] = choice_tp
#         save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="non_analogy")

#         # in this step, we only record user's decision in non-analogy case
#         return redirect(url_for('loan_task.analogy'))
#     else:
#         assert task_id < len(task_ID_list)
#         # get Loan_ID to select a data sample to show
#         loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
#         # use default task_id 'LP001002'
#         # deliver pre-task questionnaire page
#     response = render_template('loan/non_analogy.html', form=form, loan_str=loan_str, loan_data=loan_data, task_index=task_id+1)
#     # response.set_cookie('task_id', task_id)
#     return response
#     # return render_template('loan/non_analogy.html', form=form, loan_str=session['loan_str'], loan_data=session['loan_data'])

@bp.route('/accuracy', methods=('GET', 'POST'))
def accuracy():
    form = loan_form_advice()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']

    if request.method == 'POST' and form.validate_on_submit():
        # user_id = request.cookies.get('user_id')
        user_id = session['user_id']
        # task_id = g.task
        # non_analogy_answer = session['non_analogy_answer']
        # non_anlogy_time = session['non_anlogy_time']
        # analogy_answer = form.loan_choice.data
        # key_ = "analogy_choice_{}".format(task_id)
        choice_tp = form.loan_choice.data
        # session[key_] = choice_tp
        # db = get_db()
        # error = None
        # from flaskr.db import UserTask
        # ut_obj = UserTask(user_id=user_id, task_id=task_id, non_analogy_answer=non_analogy_answer, non_anlogy_time=non_anlogy_time, analogy_answer=analogy_answer)
        # db.add(ut_obj)
        # db.commit()
        save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="accuracy")
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('loan_task.base'))
        else:
            return redirect(url_for('loan_task.attention_check_2'))
            # finished, reserved_set = check_user_finish(user_id=user_id, user_group="accuracy", task_ID_list=task_ID_list)
            if finished:
                # end of tasks come to 2nd attention check question
                return redirect(url_for('loan_task.attention_check_2'))
            else:
                # To-do, continue to fill in unfinished tasks
                return redirect(url_for('loan_task.attention_check_2'))

        # flash(error)
    else:
        assert task_id < len(task_ID_list)
        assert session['last_base_task'] == task_ID_list[task_id]
        loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
    # deliver pre-task questionnaire page
    # reverse_flag_list = session['reverse_flag_list']
    # reverse_flag = eval(reverse_flag_list[task_id])
    # if reverse_flag:
    #     # reverse the answer to show
    #     system_advice = "reject" if loan_data['Loan_Status'] == "Y" else "accept"
    # else:
    #     system_advice = "accept" if loan_data['Loan_Status'] == "Y" else "reject"
    # print(len(session['system_advice_list']), len(session['system_advice_list']) == task_id)
    # if len(session['system_advice_list']) == task_id:
    #     print("save system advice into list")
    #     session['system_advice_list'].append(system_advice)
    #     print(session['system_advice_list'])
    # else:
    #     print("fail to save system advice")
    system_advice = session['system_advice_list'][task_id]
    accuracy_str = "System's accuracy is <strong>75%</strong>, and it choose to <strong>{}</strong> this application".format(system_advice)
    response = render_template('loan/accuracy.html', form=form, loan_str=loan_str, \
     loan_data=loan_data, task_index=task_id+1, accuracy_str=accuracy_str)
    return response

@bp.route('/system', methods=('GET', 'POST'))
def system():
    form = loan_form_advice()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']

    if request.method == 'POST' and form.validate_on_submit():
        # user_id = request.cookies.get('user_id')
        user_id = session['user_id']
        # task_id = g.task
        # non_analogy_answer = session['non_analogy_answer']
        # non_anlogy_time = session['non_anlogy_time']
        # analogy_answer = form.loan_choice.data
        # key_ = "analogy_choice_{}".format(task_id)
        choice_tp = form.loan_choice.data
        # session[key_] = choice_tp
        # db = get_db()
        # error = None
        # from flaskr.db import UserTask
        # ut_obj = UserTask(user_id=user_id, task_id=task_id, non_analogy_answer=non_analogy_answer, non_anlogy_time=non_anlogy_time, analogy_answer=analogy_answer)
        # db.add(ut_obj)
        # db.commit()
        save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="system")
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('loan_task.base'))
        else:
            return redirect(url_for('loan_task.attention_check_2'))
            # finished, reserved_set = check_user_finish(user_id=user_id, user_group="system", task_ID_list=task_ID_list)
            if finished:
                # end of tasks come to 2nd attention check question
                return redirect(url_for('loan_task.attention_check_2'))
            else:
                # To-do, continue to fill in unfinished tasks
                return redirect(url_for('loan_task.attention_check_2'))

        # flash(error)
    else:
        assert task_id < len(task_ID_list)
        assert session['last_base_task'] == task_ID_list[task_id]
        loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
    # deliver pre-task questionnaire page
    # reverse_flag_list = session['reverse_flag_list']
    # reverse_flag = eval(reverse_flag_list[task_id])
    # if reverse_flag:
    #     # reverse the answer to show
    #     system_advice = "reject" if loan_data['Loan_Status'] == "Y" else "accept"
    # else:
    #     system_advice = "accept" if loan_data['Loan_Status'] == "Y" else "reject"
    # print(len(session['system_advice_list']), len(session['system_advice_list']) == task_id)
    # if len(session['system_advice_list']) == task_id:
    #     # only append for the first time
    #     session['system_advice_list'].append(system_advice)
    system_advice = session['system_advice_list'][task_id]
    system_str = "System chooses to <strong>{}</strong> this application.".format(system_advice)
    response = render_template('loan/system.html', form=form, loan_str=loan_str, \
     loan_data=loan_data, task_index=task_id+1, system_str=system_str)
    return response

@bp.route('/analogy', methods=('GET', 'POST'))
def analogy():
    form = loan_form_advice()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    analogy_type = session['analogy_type']

    if request.method == 'POST' and form.validate_on_submit():
        # user_id = request.cookies.get('user_id')
        user_id = session['user_id']
        # task_id = g.task
        # non_analogy_answer = session['non_analogy_answer']
        # non_anlogy_time = session['non_anlogy_time']
        # analogy_answer = form.loan_choice.data
        # key_ = "analogy_choice_{}".format(task_id)
        choice_tp = form.loan_choice.data
        # session[key_] = choice_tp
        # db = get_db()
        # error = None
        # from flaskr.db import UserTask
        # ut_obj = UserTask(user_id=user_id, task_id=task_id, non_analogy_answer=non_analogy_answer, non_anlogy_time=non_anlogy_time, analogy_answer=analogy_answer)
        # db.add(ut_obj)
        # db.commit()
        save_choice(user_id, task_ID_list[task_id], choice_tp, answer_type="analogy")
        if task_id < len(task_ID_list) -1:
            # inser before redirection to next page
            session['task_id'] += 1
            # start next task
            return redirect(url_for('loan_task.base'))
        else:
            return redirect(url_for('loan_task.attention_check_2'))
            # finished, reserved_set = check_user_finish(user_id=user_id, user_group="analogy", task_ID_list=task_ID_list)
            if finished:
                # end of tasks come to 2nd attention check question
                return redirect(url_for('loan_task.attention_check_2'))
            else:
                # To-do, continue to fill in unfinished tasks
                return redirect(url_for('loan_task.attention_check_2'))

        # flash(error)
    else:
        assert task_id < len(task_ID_list)
        assert session['last_base_task'] == task_ID_list[task_id]
        loan_str, loan_data = fetch_task_info(task_ID_list[task_id])
    if analogy_type == "vaccine":
        analogy_str = vaccine_analogy
    elif analogy_type == "train":
        analogy_str = train_analogy
    else:
        analogy_str = weather_analogy
    # deliver pre-task questionnaire page
    # reverse_flag_list = session['reverse_flag_list']
    # reverse_flag = eval(reverse_flag_list[task_id])
    # if reverse_flag:
    #     # reverse the answer to show
    #     system_advice = "reject" if loan_data['Loan_Status'] == "Y" else "accept"
    # else:
    #     system_advice = "accept" if loan_data['Loan_Status'] == "Y" else "reject"
    # print(len(session['system_advice_list']), len(session['system_advice_list']) == task_id)
    # if len(session['system_advice_list']) == task_id:
    #     print("save system advice into list")
    #     session['system_advice_list'].append(system_advice)
    #     print(session['system_advice_list'])
    # else:
    #     print("fail to save system advice")
    system_advice = session['system_advice_list'][task_id]
    analogy_str = "{}, and it choose to <strong>{}</strong> this application".format(analogy_str, system_advice)
    response = render_template('loan/analogy.html', form=form, loan_str=loan_str, \
     loan_data=loan_data, task_index=task_id+1, analogy_str=analogy_str)
    return response


# @bp.before_app_request
# def load_logged_in_user():
#     user_id = session.get('user_id')

#     if user_id is None:
#         g.user = None
#     else:
#         g.user = user_id

# def login_required(view):
#     @functools.wraps(view)
#     def wrapped_view(**kwargs):
#         if g.user is None:
#             # we may define an error page
#             return redirect(url_for('auth.login'))

#         return view(**kwargs)

#     return wrapped_view

