from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, SelectField, RadioField
from wtforms.validators import DataRequired, Length, StopValidation


template_list = ["Select Template", "Definite Sign Of", "Typically Associated With", "Seldom Found At", "Contradict With", "Insufficient", "Irrelevant"]
template_choices = [("0", "Select a template"), ("1", "Definite Sign Of"), ("2", "Typically Associated With"), ("3", "Seldom Found At"), ("4", "Contradict With"), ("5", "Insufficient"), ("6", "Irrelevant")]
id2template = {
    "0": "Select a template",
    "1": "Definite Sign Of",
    "2": "Typically Associated With",
    "3": "Seldom Found At",
    "4": "Contradict With",
    "5": "Insufficient",
    "6": "Irrelevant"
}

stop_phrase_place = ["living room", "bathroom", "hospital room", "conference room", "bedroom", "dining room", "kitchen", "cooker and oven", 
"oven", "towel and shampoo", "towel", "shampoo", "toilet", "medical monitor", "toothbrush and towel", "window and curtain", "table and chair", "fireplace", "hospital", "ceiling"]

def check_overlap_places(temp_str):
    new_str = temp_str.lower()
    overlap_list = []
    for phrase in stop_phrase_place:
        if phrase in new_str:
            overlap_list.append(phrase)
    overlap_str = None
    if len(overlap_list) > 0:
        overlap_str = "Error, you used concepts in places and furniture domain or the task: {}! You are supposed to generate analogy-based explanation in another domain.".format("; ".join(overlap_list))
    return overlap_str

stop_phrase_calorie = ["mayonnaise", "calorie", "high calorie", "low calorie", "trunk", "elephant", "chocolate", "printer", "office", "home", "bread", "chair",
"living room", "bathroom", "hospital room", "conference room", "bedroom", "dining room", "kitchen", "plate", "arbitrary stone", "continent", "carrot", "cat", "water", "meat", "vegetable", "salad"]


def check_overlap_calorie(temp_str):
    new_str = temp_str.lower()
    overlap_list = []
    for phrase in stop_phrase_calorie:
        if phrase in new_str:
            overlap_list.append(phrase)
    overlap_str = None
    if len(overlap_list) > 0:
        overlap_str = "Error, you used concepts in food and calorie domain or the task: {}! You are supposed to generate analogy-based explanation in another domain.".format("; ".join(overlap_list))
    return overlap_str


class OptionRequired(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """
    field_flags = ('required', )

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        # print(field.data)
        if field.data == '0':
            if self.message is None:
                message = field.gettext('Must choose an option in this field.')
            else:
                message = self.message

            field.errors[:] = []
            raise StopValidation(message)

class ConceptPlaceForbid(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """
    # field_flags = ('required', )

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        current_str = field.data
        overlap_str = check_overlap_places(current_str)
        if overlap_str is not None:
            message = field.gettext(overlap_str)
            field.errors[:] = []
            raise StopValidation(message)
        tokens = current_str.strip().split(" ")
        if len(tokens) > 5:
            message = field.gettext("You used more than 5 words in this field")
            field.errors[:] = []
            raise StopValidation(message)

class ConceptCalorieForbid(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """
    # field_flags = ('required', )

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        current_str = field.data
        overlap_str = check_overlap_calorie(current_str)
        if overlap_str is not None:
            message = field.gettext(overlap_str)
            field.errors[:] = []
            raise StopValidation(message)
        tokens = current_str.strip().split(" ")
        if len(tokens) > 5:
            message = field.gettext("You used more than 5 words in this field")
            field.errors[:] = []
            raise StopValidation(message)

class ConceptCRequired_places(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """
    # field_flags = ('required', )

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        template_str = form.template.data
        template_name = id2template[template_str]
        if template_name in ["Typically Associated With", "Insufficient"]:
            if len(form.concept_3.data) < 1:
                if self.message is None:
                    message = field.gettext('For {} template, you must fill in ConceptC'.format(template_name))
                else:
                    message = self.message
                
                field.errors[:] = []
                raise StopValidation(message)
            else:
                current_str = form.concept_3.data
                overlap_str = check_overlap_places(form.concept_3.data)
                if overlap_str is not None:
                    message = field.gettext(overlap_str)
                    field.errors[:] = []
                    raise StopValidation(message)
                tokens = current_str.strip().split(" ")
                if len(tokens) > 5:
                    message = field.gettext("You used more than 5 words in this field")
                    field.errors[:] = []
                    raise StopValidation(message)


class ConceptCRequired_calorie(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """
    # field_flags = ('required', )

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        template_str = form.template.data
        template_name = id2template[template_str]
        if template_name in ["Typically Associated With", "Insufficient"]:
            if len(form.concept_3.data) < 1:
                if self.message is None:
                    message = field.gettext('For {} template, you must fill in ConceptC'.format(template_name))
                else:
                    message = self.message
                
                field.errors[:] = []
                raise StopValidation(message)
            else:
                current_str = form.concept_3.data
                overlap_str = check_overlap_calorie(form.concept_3.data)
                if overlap_str is not None:
                    message = field.gettext(overlap_str)
                    field.errors[:] = []
                    raise StopValidation(message)
                tokens = current_str.strip().split(" ")
                if len(tokens) > 5:
                    message = field.gettext("You used more than 5 words in this field")
                    field.errors[:] = []
                    raise StopValidation(message)

class submit_form(FlaskForm):
    submit_button = SubmitField('Next')


